'use strict';
/*
 * This job renders `data` using nunjucks template `type`
 */

var nunjucks = require('nunjucks');
var fs = require('fs');
var dateFilter = require('nunjucks-date-filter');
var util = require('util');

var watch = require('./config').watch;
console.log('watch: '+watch);
var env = new nunjucks.Environment(
  new nunjucks.FileSystemLoader('theme/templates',{watch: watch})
);

env.addFilter('date', dateFilter);

env.addFilter('paginationList', function(page, count) {
    var curPage = page;
    var pages = [];
    pages.push(curPage);
    for(var i = 0; i < count; i++) {
      curPage = curPage.metadata.prevpage;
      if(curPage === undefined) break;
      pages.unshift(curPage);
    }

    curPage = page;
    for(let i = 0; i < count; i++) {
      curPage = curPage.metadata.nextpage;
      if(curPage === undefined) break;
      pages.push(curPage);
    }
    return pages;
});

env.addFilter('inspect', function(obj) {
  return util.inspect(obj);
});

env.addFilter('log', function(obj) {
  return console.log(obj);
});

env.addFilter('relURL', function(filename, dir){
  if(filename.substr(0,1) == '/' || filename.match('://')) return filename;
  if(dir.substr(0,1) !== '/') dir = '/' + dir;
  if(filename.match(/\/$/)) return dir+filename;
  return dir+'/'+filename;
})

var formats = {};

module.exports = function(data, type) {
  return new Promise(function(resolve, reject) {
    if(formats[type]) resolve(formats[type]);
    else {
      fs.readdir('theme/templates',function(err, files) {
        if(err) return reject(new Error('Error listing files in theme/templates'));

        files = files.filter(function(file) {
          return file.match(type+'\..*\.nunj');
        });

        var filteredFiles = [];
        var todo = 0;
        files.forEach(function(file) {
          fs.stat('theme/templates/'+file,function(err, stat) {
            if(err) reject(new Error('Error stating theme/templates/'+file));
            if(stat.isFile())
              filteredFiles.push(file);
            todo--;
            if(todo <= 0) {
              formats[type] = filteredFiles;
              resolve(filteredFiles);
            }
          });
          todo++;
        });
      });
    }
  }).then(function(filelist) {
    var promises = [];
    filelist.forEach(function(file){
      var type = file.replace(/^.*\.([a-z]+)\.nunj/,'$1');
      promises.push(new Promise(function(resolve, reject) {
        env.getTemplate(file, function(err, tmpl) {
          if(err) reject(new Error('Failed to load template '+type+'.html.nunj'));
          resolve({tmpl: tmpl, type: type, file: file});
        });
      }));
    });
    return Promise.all(promises);
  }).then(function(tmpls) {
    var promises = [];
    tmpls.forEach(function(o) {
      promises.push(new Promise(function(resolve, reject){
        o.tmpl.render(data, function(err, val) {
          if(err) {
            //reject(new Error('Error rendering template '+o.file));
            reject(err);
          }
          resolve({content: val, type: o.type});
        });
      }));
    });
    return Promise.all(promises);
  });
}

module.exports.reset = function() {
  formats = {};
}
